
/*
 --   ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      b004link.c
 --
 --      Link module for B004 type boards in IBM PCs running Linux
 --
 --      Copyright (c) INMOS Ltd., 1988.
 --      Copyright (c) Christoph Niemann, 1993
 --      All Rights Reserved.
 --
 --   ---------------------------------------------------------------------------
*/

#include <linux/b004.h>
#include <fcntl.h>

#include "inmos.h"
#include "iserver.h"


#define NULL_LINK -1

PRIVATE LINK ActiveLink = NULL_LINK;

/*
 *   Open Link
 */

PUBLIC LINK OpenLink ( Name )
   char *Name;
{
   
   if ( ActiveLink != NULL_LINK )
      return( ER_LINK_CANT );
   if ( *Name == '\0')
   {
     if ( ( ActiveLink = open( "/dev/link" , O_RDWR ) ) < 0 )
        return( ER_LINK_BUSY );
   }
   else
   {
     if ( ( ActiveLink = open( Name , O_RDWR ) ) < 0 )
        return( ER_LINK_BUSY );
   }

   return(ActiveLink);
}




/*
 *   Close Link
 */

int CloseLink ( LinkId )
   int LinkId;
{
   if ( LinkId != ActiveLink )
      return( -1 );
   close(ActiveLink);
   ActiveLink = NULL_LINK;
   return(TRUE);
}





/*
 *   Read Link
 */

int ReadLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  return (read(LinkId, Buffer, Count));
}




/*
 *   Write Link
 */

int WriteLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  return (write(LinkId, Buffer, Count));
}



/*
 *   Reset Link
 */

PUBLIC int ResetLink ( LinkId )
   LINK LinkId;
{

   if ( LinkId != ActiveLink )
      return( -1 );
   if ( ioctl( LinkId, B004RESET, 1) != 1 )
      return( -1 );

   return( 1 );
}



/*
 *   Analyse Link
 */
 
PUBLIC int AnalyseLink ( LinkId )
   LINK LinkId;
{

   if ( LinkId != ActiveLink )
      return( -1 );
   if ( ioctl( LinkId, B004ANALYSE, 0) != 0 )
      return( -1 );

   return( 1 );
}



/*
 *   Test Error
 */
 
PUBLIC int TestError ( LinkId )
   LINK LinkId;
{
   if ( LinkId != ActiveLink )
      return( -1 );

   return ( ioctl( LinkId, B004ERROR, 0 ) );
}




/*
 *   Test Read
 */

PUBLIC int TestRead ( LinkId )
   LINK LinkId;
{
   if ( LinkId != ActiveLink )
      return( -1 );

   return ( ioctl( LinkId, B004READABLE, 0 ) );
}




/*
 *   Test Write
 */

PUBLIC int TestWrite ( LinkId )
   LINK LinkId;
{
   if ( LinkId != ActiveLink )
      return( -1 );

   return ( ioctl( LinkId, B004WRITEABLE, 0 ) );
}



/*
 *   Eof
 */


