/*
 * claim.c
 *
 * Code to construct a Meiko svcs proc/wire claim structure from
 * arguments passed to a program.
 *
 * (C) David Beckett 1991 University of Kent at Canterbury
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <svcs.h>
#include "inmos.h"
#include "iserver.h"
#include "types.h"
EXTERN BOOL CocoPops,VerboseSwitch;
EXTERN LINK TheLink;
EXTERN Resources theResources;

/* **********************************************************************
 * claim
 * Claim the given processors in the list of matches.
 * **********************************************************************
 */

struct DomainSpec *
claim(matches, wires, wait, vcsfd, supfd)
  List *matches;
  List *wires;
  BOOL wait;
  int *vcsfd;
  int *supfd;
{
  int result;
  struct ProcessorSpec *p;
  struct DomainSpec *theDomain;
  Match *n;
  Wire *w;
  struct Processor *pr;
  int i;

  /* Initialise the domain */
  theDomain = svcs_newspec();
  p= theDomain->d_spec;

  if(CocoPops)
    fprintf(stderr,"REPORT ON PROCESSORS\n");

  /* Claim the processors */
  for(n=(Match*)matches->head; n!=NULL; n=(Match*)n->node.next)
    {
      if(CocoPops)
	print_match(stderr, n);

      /* Repeat for given count */
      for(i=0; i< n->count; i++)
	{
          if (n->proc_type == ANY)
	    svcs_procspec(theDomain, ANY, 0);  /* THIS IS BUGGED */
          else
	    svcs_procspec(theDomain, T800, 8);  /* THIS IS BUGGED */
	  switch(n->proc_type)
	    {
	    case MATCH_INT_NONE:
	      p->p_types=ANY;
	      break;
	    case 414:
	      p->p_types=T414;
	      break;
	    case 800:
	      p->p_types=T800;
	      break;
	    case 425:
	      p->p_types=T425;
	      break;
	    case 801:
	      p->p_types=T801;
	      break;
	    case ANY:
	      break;
	    default:
	      ABORT(MISC_EXIT, (SE," Illegal processor type t%d\n", n->proc_type));
	    }
	  
	  /* memory size needed in MEMUNITs */
	  if(n->proc_memory==MATCH_INT_NONE)
	    p->p_memunits=1;
	  else
	    p->p_memunits=n->proc_memory/256; /* Convert K to MEMUNITs */
	  
	  /* MKxxx board type code, or 0xff for any */
	  if(n->board_type==MATCH_INT_NONE)
	    p->p_boardtype=ANY;
	  else
	    p->p_boardtype=n->board_type;
	  
	  /* module, or 0xff for any */
	  if(n->module==MATCH_INT_NONE)
	    p->p_module=ANY;
	  else
	    p->p_module=n->module;
	  
	  /* board,  or 0xff for any */
	  if(n->board==MATCH_INT_NONE)
	    p->p_board=ANY;
	  else
	    p->p_board=n->board;
	  
	  /* proc,   or 0xff for any */
	  if(n->processor==MATCH_INT_NONE)
	    p->p_proc=ANY;
	  else
	    p->p_proc=n->processor;
	  p++;
	}
    }

  if(wait)
    theDomain->d_wait=1;
  else
    theDomain->d_wait=0;
  /* Now test if any processors are requested at all */
  /* because if there are none, we should not call svcs_grab (it crashes) */
  if (theDomain->d_nspecs < 2) {
    /* add processors and wires */
    ABORT(MISC_EXIT,(SE,"No processor allocated!\n")); 
  }
  /* Wire up the domain */
  for(w=(Wire*)wires->head; w!=NULL; w=(Wire*)w->node.next){
    DEBUG(("Adding wire %d.%d.%d.%d", w->p0, w->l0, w->p1, w->l1));
    /* test on validity of processor numbers */
    if (w->p0 >= theDomain->d_nspecs) 
        ABORT(MISC_EXIT,(SE,"Error in wire spec: No such processor %d",w->p0));
    if (w->p1 >= theDomain->d_nspecs) 
        ABORT(MISC_EXIT,(SE,"Error in wire spec: No such processor %d",w->p1));
    svcs_wirespec(theDomain, w->p0, w->l0, w->p1, w->l1);
  }
  if(CocoPops)
    {
      fprintf(stderr,"%s: REPORT ON DOMAIN CLAIM\n", PROGRAM_NAME);

      fprintf(stderr,"%s: Pid %5d Gid %5d\n",
	      PROGRAM_NAME, theDomain->d_pid, theDomain->d_gid);
      
      fprintf(stderr,"%s: Lifetime %5d Priority %5d Wait %5d\n",
	      PROGRAM_NAME, theDomain->d_lifetime, theDomain->d_priority,
	      theDomain->d_wait);

      fprintf(stderr,"%s: No.Specs: %5d No.Wires: %5d\n",
	      PROGRAM_NAME, theDomain->d_nspecs, theDomain->d_nwires);
   }

  result = svcs_grab(theDomain, vcsfd, supfd, &theResources);
  
  if(CocoPops)
    {
      fprintf(stderr,"%s: REPORT ON CLAIMED RESOURCES\n", PROGRAM_NAME);
      
      fprintf(stderr,"%s: p1 = %d p2 = %d p3 = %d\n", PROGRAM_NAME,
	      theResources.r_p1, theResources.r_p2, theResources.r_p3);
      
      fprintf(stderr,"%s: rc = %d linkname = %s\n", PROGRAM_NAME,
	      theResources.r_rc, theResources.r_linkname);
      
      fprintf(stderr,"%s: board = %d link = %d nprocs = %d\n",
	      PROGRAM_NAME, theResources.r_board, theResources.r_link,
	      theResources.r_nprocs);
      
      pr=theResources.r_proc;
      for(i=0;i < theResources.r_nprocs; pr++, i++)
	fprintf(stderr,
		"%s: Processor %d - Type %d Memunits %3d (%4dK) SVBusID %3d\n",
		PROGRAM_NAME, i, pr->p_type,
		pr->p_memunits, 256*pr->p_memunits,
		pr->p_svbusid);
    }
  
  if (result<0)
    {
      fprintf(stderr,"%s: Failed to claim processors (%s)\n",
		  PROGRAM_NAME, svcs_errmsg(svcs_errno));
      theDomain=NULL;
    }

  return(theDomain);
  
}

/* **********************************************************************
 * free_claim
 * Free the earlier claim for processors in the list of matches.
 * **********************************************************************
 */

void
free_claim(d)
  struct DomainSpec *d;
{
  svcs_freespec(d);
}

/* **********************************************************************
 * get_wire
 * Parse the wire specification
 * Set munge=TRUE to add 1 to all proc. numbers to account for the 
 * intermediate buffer processor.
 * **********************************************************************
 */

BOOL
get_wire(arg, wires, munge)
  char *arg;
  List *wires;
  BOOL munge;
{
  Wire *w;
  FILE *wire_file;
  char wire_buffer[WIRE_BUFFER_SIZE];
  int rc;
  int no_wires;
  int i;
  
  if(CocoPops)
    fprintf(stderr,"REPORT ON WIRES\n");
  
  /* Allocate first wire */
  w=(Wire*)malloc((int)sizeof(Wire));
  if(!w)
    ABORT(MISC_EXIT, (SE, "Out of memory"));

  rc=sscanf(arg, "%d%*[^0-9]%d%*[^0-9]%d%*[^0-9]%d",
	    &w->p0, &w->l0, &w->p1, &w->l1);
  if(rc==4) { /* OK */
    if(CocoPops)
      fprintf(stderr,"%s: Found wire p%d l%d to p%d l%d\n",
	      PROGRAM_NAME, w->p0, w->l0, w->p1, w->l1);

    if(munge)
      {
	w->p0++;
	w->p1++;
      }
    add_tail(wires, (Node*)w);
    return(FALSE);
  }

/* 
  if(rc>0)
    {
      fprintf(stderr,"%s: Illegal wire spec %s\n", PROGRAM_NAME, arg);
      free((void*)w);
      return(TRUE);
    }
 */
  if(!(wire_file=fopen(arg, "r")))
    {
      fprintf(stderr, "%s: Cannot read wiring file %s\n", PROGRAM_NAME, arg);
      free((void*)w);
      return(TRUE);
    }
  
  fgets(wire_buffer, WIRE_BUFFER_SIZE, wire_file);
  rc=sscanf(wire_buffer, "%d;", &no_wires);
  if(CocoPops)
    fprintf(stderr,"%s: %d wires found\n", PROGRAM_NAME, no_wires);
  
  for(i=0; i<no_wires && !feof(wire_file); i++)
    {
      w=(Wire*)malloc((int)sizeof(Wire));
      if(!w)
	ABORT(MISC_EXIT, (SE, "Out of memory"));

      fgets(wire_buffer, WIRE_BUFFER_SIZE, wire_file);
      rc=sscanf(wire_buffer, "%d%*[^0-9]%d%*[^0-9]%d%*[^0-9]%d",
		&w->p0, &w->l0, &w->p1, &w->l1);
      if(CocoPops)
	fprintf(stderr,"%s: Found wire p%d l%d to p%d l%d\n",
		PROGRAM_NAME, w->p0, w->l0, w->p1, w->l1);
      if(munge)
	{
	  w->p0++;
	  w->p1++;
	}
      if(rc!=4)
	{
	  fprintf(stderr,"%s: Illegal wire spec %s\n",
		  PROGRAM_NAME, wire_buffer);
	  free((void*)w);
	  fclose(wire_file);
	  return(TRUE);
	}
      else
	add_tail(wires, (Node*)w);
    }
  fgets(wire_buffer, WIRE_BUFFER_SIZE, wire_file);
  if(!feof(wire_file))
    fprintf(stderr,"%s WARNING: Extra data after end of wiring file %s\n",
	    PROGRAM_NAME, arg);
  fclose(wire_file);
  return(FALSE);
}

