/* **********************************************************************
 *
 * node.h
 *
 * Plug in doubly-linked list routines
 *
 * (C) David Beckett 1991, University of Kent at Canterbury
 * **********************************************************************
 */

#include <stdio.h>
#include "node.h"

/* **********************************************************************
 * new_list - Empty the list
 * **********************************************************************
 */
void
new_list (list)
  List *list;
{
  list->head=NULL;
  list->tail=NULL;
  return;
}

/* **********************************************************************
 * add_head - Add the given node to the head of the list.
 * **********************************************************************
 */

void
add_head (list, node)
  List *list;
  Node *node;
{
  if(list->head==NULL)
    {
      list->head=node;
      list->tail=node;
      node->next=NULL;
      node->prev=NULL;
    }
  else
    {
      node->prev=NULL;
      node->next=list->head;
      list->head->prev=node;
      list->head=node;
    }
  return;
}

/* **********************************************************************
 * add_tail - Add the given node to the tail of the list.
 * **********************************************************************
 */

void
add_tail (list, node)
  List *list;
  Node *node;
{
  if(list->tail==NULL)
    {
      list->head=node;
      list->tail=node;
      node->next=NULL;
      node->prev=NULL;
    }
  else
    {
      node->next=NULL;
      node->prev=list->tail;
      list->tail->next=node;
      list->tail=node;
    }
  return;
}

/* **********************************************************************
 * del_head - Delete the head node, and free it.
 * **********************************************************************
 */

void
del_head (list)
  List *list;
{
  Node *temp;
  
  if (list->head==NULL)
    return;
  temp=list->head;
  temp->next->prev=NULL;
  list->head=temp->next;
  if(list->head==NULL)
    list->tail=NULL;
  free((char*)temp);
  return;
}

/* **********************************************************************
 * del_tail - Delete the tail node, and free it.
 * **********************************************************************
 */

void
del_tail (list)
  List *list;
{
  Node *temp;

  if (list->tail==NULL)
    return;
  temp=list->tail;
  temp->prev->next=NULL;
  list->tail=temp->prev;
  if(list->tail==NULL)
    list->head=NULL;
  free((char*)temp);
  return;
}

/* **********************************************************************
 * del_tail - Delete the tail node, and free it.
 * **********************************************************************
 */

void
free_list (list)
  List *list;
{
  Node *temp;
  
  if(list->head==NULL)
    return;
  temp=list->head;
  while(temp!=NULL)
    {
      list->head=temp->next;
      free((char*)temp);
      temp=list->head;
    }
  list->tail=temp;
  return;
}

