/*
 * skeleton.c
 *
 * Just claim the procs asked for - no questions asked; then wait for
 * a signal.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "/usr/meiko/vcs/include/svcs.h"
#include "types.h"
#include "proto.h"
#include "claim-proto.h"
#include "version.h"

/* **********************************************************************
 * Global variables
 * **********************************************************************
 */

char *progname;   /* The name of this program (basename) */
List matches;     /* The linked list of Matches          */
List wires;       /* The linked list of wires            */
Bool debug;       /* Debug flag                          */

/* **********************************************************************
 * Constants for here
 * **********************************************************************
 */

/* **********************************************************************
 * Main Program
 * **********************************************************************
 */

void main(int argc, char *argv[])
{
  Bool arrgh_error;       /* Argument error flag                            */
  Bool usage;             /* Usage information flag                         */
  Bool wait;              /* Wait for processor claim                       */
  int c;                  /* Char returned by getopt(3)                     */
  extern char *optarg;
  FILE *output;
  Match *a;
  struct DomainSpec *d;


  /* *************************************
   * Initialise global variables
   * *************************************
   */
  progname=basename(argv[0]); /* Find out who I am */
  debug=FALSE;

  /* Empty the lists used in here */
  new_list(&matches);
  new_list(&wires);

  /* *************************************
   * Parse Matches
   * *************************************
   */
  
  /* Initialise defaults */
  output=stdout;  /* Output goes to stdout                           */
  wait=TRUE;      /* Normally wait for processor claim               */
  d=NULL;

  /* No Match error so far ... */
  arrgh_error=FALSE;
  usage=FALSE;

  /* Use getopt(3), for the moment */
  while((c=getopt(argc, argv, TESTARGS))!= -1)
    switch (c) 
      {
      case 'd':
	debug=TRUE;
	break;
      case 'h':
	usage=TRUE;
	break;
      case 'i':
	wait=FALSE;
	break;
      case 'p':
	/* Processor Match - parse it and add it to linked list */
	a=(Match*)malloc((int)sizeof(Match));
	if(!a)
	  panic("Out of memory");
	if(get_match(optarg,a))
	  free((void *)a);
	else
	  add_tail(&matches, (Node*)a);
	break;
      case 'w':
	/* Wire - parse it and add it to linked list */
	if(get_wire(optarg, &wires))
	  arrgh_error=TRUE;
	break;
      case '?':
      default:
	arrgh_error=TRUE;
      }
  
  /* Something went wrong */
  if (arrgh_error)
    {
      fprintf(stderr,"Usage: %s [-%s]\n (Type %s -h for help)\n",
	      progname,TESTARGS,progname);
      do_exit(0);
    }
  /* If invoked for help, give the full usage information */
  if (usage)
    {
      help(stderr);
      do_exit(0);
    }

  if(matches.head!=NULL)
    d=claim(&matches, &wires, wait, output);

  /* If grabbed - do something and clean up */
  if(d)
    {
      sleep(10);
      free_claim(d);
    }
  
  /* Clean termination */
  do_exit(0);
  
}

/* **********************************************************************
 * Basename
 * Return the base file name (ie no path) of the given filename.
 * **********************************************************************
 */

char *basename(char *name)
{
  char *p;

  p=name+strlen(name)-1;
  while((p>name)&&*p!='/') p--;
  if (p==name) return name;
  return(p+1);
}

/* **********************************************************************
 * panic
 * Fall out of code in middle with a message; should never happen as
 * they say
 * **********************************************************************
 */

void panic (char *message)
{
  fprintf(stderr,"%s: FATAL ERROR, %s\n",progname, message);
  do_exit(10);
}

/* **********************************************************************
 * do_exit - do a clean exit, closing down as needed.
 * **********************************************************************
 */

void do_exit(int error_code)
{
  if (matches.head!=NULL)
    free_matches(&matches);
  if (error_code>=0)
    exit(error_code);
}

/* **********************************************************************
 * help - print the usage information
 * **********************************************************************
 */

void help(FILE *stream)
{
  fprintf(stream,"%s V%s %s by David Beckett (djb1@ukc.ac.uk)\n",
	  progname,VERSION,__DATE__);
  fprintf(stream,"Usage: %s [-hiv] [-p <proc-spec>] [-w <wiring>]\n",progname);
  fprintf(stream,"-p <proc-spec> The processor claim eg\n");
  fprintf(stream,"  -p mk015 -p t8/4m -p m2b1p0 -p b1 -p 2xt8/4m/b7\n");
  fprintf(stream,"-w <wiring>    The wiring eg\n");
  fprintf(stream,"  -w 0.1.2.2  -w test.wir  -w 10.3.1.2\n");
  fprintf(stream,"\n");
  fprintf(stream,"Other options:\n");
  fprintf(stream," -d Show debugging information on claiming\n");
  fprintf(stream," -h Show help information (this)\n");
  fprintf(stream," -i Do not wait for processors to be freeded\n");
}

