/************************ supercmd.h **************************/

#ifndef supercmd_h
#define supercmd_h 1
/*
 * Gross supervisor protocol.
 */
#define TYPE_T414 0
#define TYPE_T800 1
#define TYPE_T425 2
#define TYPE_T801 3

#define SUPER_MSGTYPE    0
#define SUPER_MSGDATA    1
#define SUPER_BADID      -1
#define SUPER_INFCOST    0x7fff

#define SUPER_MAXMESSAGE 256
#define SUPER_MAXENTRY   (sizeof(SuperMsg) / sizeof(int))

typedef struct SuperMsg {
    int  super_cmd;
    int  super_id;
    int  super_p1;
    int  super_p2;
    int  super_p3;
    int  super_p4;
    char data[SUPER_MAXMESSAGE];
} SuperMsg;

#define SUPER_ENTRY 6

/*
 * Supervisor protocol commands.
 */ 
#define SUPER_SBREAD	    0
#define SUPER_SBWRITE	    1
#define SUPER_RESET	    2
#define SUPER_ANALYSE	    3
#define SUPER_SEND	    4
#define SUPER_ENABLE	    5
#define SUPER_DISABLE	    6
#define SUPER_STARTMONITOR  7
#define SUPER_ENDMONITOR    8
#define SUPER_SOFTERROR     9
#define SUPER_HARDERROR    10
#define SUPER_RECEIVE      11
#define SUPER_NPROCESSORS  12
#define SUPER_PLACEMENT    13
#define SUPER_INIT         14
#define SUPER_REPORTREQ    15
#define SUPER_GETID        16
#define SUPER_SETSTATE     17
#define SUPER_RESETSTATE   18
#define SUPER_STOP	   19
#define SUPER_STOPONHARDERROR  20
#define SUPER_TEST         21
#define SUPER_ADDRERROR    22
#define SUPER_ADDRMAP      23
#define SUPER_PROCESSORTYPE    24
#define SUPER_FINISH       25
#define SUPER_CONNECT      26
#define SUPER_DISCONNECT   27
#define SUPER_COST         28
#define SUPER_FCONNECT     29
#define SUPER_TRISTATEALL  30
#define SUPER_RESERVEROUTE 31

/*
 * Fields in supervisor protocol messages.
 */
#define enable_enableType  super_p1
 
#define super_module       super_p1
#define super_board        super_p2
#define super_proc         super_p3
#define super_type         super_p4

#define message_count      super_p1
#define message_message    super_p4

#define state_state        super_p1

#define addrmap_ramBase    super_p1
#define addrmap_intRamTop  super_p2
#define addrmap_extRamTop  super_p3

#define proctype_processor super_p1

#define rt_linkA           super_p1
#define rt_linkB           super_p2
#define rt_procB           super_p3
#define rt_route           super_p4
#define rt_cost            super_p1
#define rt_ways            super_p2
#define rt_sort            super_p1

/*
 * Consts for packing routing information.
 */
#define RT_VIASHIFT    0
#define RT_VIAMASK     0x01
#define RT_HCHANSHIFT  1
#define RT_HCHANMASK   0x1f
#define RT_VCHANSHIFT  6
#define RT_VCHANMASK   0x07

/*
 * Abstract supervisor bus functions understood by SBREAD/SBWRITE.
 */
#define FN_RESET	     0
#define FN_ANALYSE	     1
#define FN_DATAREAD	     2
#define FN_DATAWRITE	     3
#define FN_SLAVEOUTFULL      4
#define FN_SLAVEINFULL       5
#define FN_SLAVECAUSEDINT    6
#define FN_CAUSESLAVEINT     7
#define FN_REMOVEMASTERINT   8
#define FN_ROUTER	     9
#define FN_SOFTERROR	    10
#define FN_HARDERROR        11
#define FN_ADDRERROR	    12
#define FN_ENABLESLAVEINT   13
#define FN_ENABLEHARDINT    14
#define FN_ENABLESOFTINT    15
#define FN_HARDERRORBYTE    16
#define FN_HARDERRORBANK    17
#define FN_NFUNCTIONS       18

#endif

/******************* end header file "supercmd.h" ************************/ 

