
/*
 --   ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      svcslink.c
 --
 --      Link module for Meiko boards with /dev/mpn device driver
 --
 --      Copyright (c) H7 1990.
 --      All Rights Reserved.
 --
 --   ---------------------------------------------------------------------------
*/

#include <memory.h>
#include <stdio.h>
#include <fcntl.h>
#include "svcs.h"
#include "inmos.h"
#include "iserver.h"
#include "supercmd.h"

EXTERN LINK TheLink;
PUBLIC int vcsfd, supfd, CocoPops;
EXTERN BOOL VerboseSwitch;
PUBLIC Resources theResources;
PUBLIC int vcsfd;
PUBLIC int supfd;
PUBLIC DomainSpec *theDomain;
PRIVATE BYTE *BufferBootFileName;



/*
 *   Open Link
 */

PUBLIC LINK OpenLink ( Name )
   BYTE *Name;
{
  int result, cprfd;
  long FileSize;
  struct ProcessorSpec *p;

#ifdef SVCS
  theDomain = svcs_newspec();
  p= theDomain->d_spec;
  svcs_procspec(theDomain, T800, 1); /* processor which just forwards messages */
  p++;
  svcs_procspec(theDomain, T800, 16); /* processor doing all the work */
  p->p_memunits = 4 *MEGABYTES;
  p++;
  svcs_procspec(theDomain, T800, 4); /* processor for loading PROGRAMS on*/
  p->p_memunits = 1 *MEGABYTES;
  svcs_wirespec(theDomain, 0, 2, 1, 0);
  svcs_wirespec(theDomain, 1, 1, 2, 2); /* default OPS connection: -w 0.1.1.2 */
  if ((result = svcs_grab(theDomain, &vcsfd, &supfd, &theResources)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Grabbed %d processors", theResources.r_nprocs)) ;  
  DEBUG(("Processor 0 is %s", theResources.r_linkname));
  DEBUG(("Supv busid of worker is %d", theResources.r_proc[1].p_svbusid));
  if ( ( BufferBootFileName = (BYTE *)getenv("IBUFFER") ) == NULL )
    ABORT(MISC_EXIT,(SE, "No environment variable IBUFFER defined"));
  if ((cprfd = open(BufferBootFileName, O_RDONLY)) == -1)
    ABORT(MISC_EXIT,(SE, "Failed to open buffer boot file %s", BufferBootFileName));
  if ((result = svcs_bootcpr(vcsfd, cprfd)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Buffer process booted using %s", BufferBootFileName)) ;  
#endif

   return(1);
}




/*
 *   Close Link
 */

PUBLIC int CloseLink ( LinkId )
   LINK LinkId;
{
   return(1);
}




/*
 *   Read Link
 */
 
PUBLIC int ReadLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result, RecvCount;
  BYTE *RecvPtr = (BYTE *) Buffer;

   DEBUG(("ReadLink requests %d bytes",Count)); 
#ifdef SVCS
  { BYTE Buf[4]; register BYTE *c; 
    RecvCount = -Count;
         c = (BYTE *)&RecvCount;
#ifdef BIG_ENDIAN
         Buf[3]= *c++ ; Buf[2]= *c++ ; Buf[1]= *c++ ; Buf[0]= *c;
#else
         Buf[0]= *c++ ; Buf[1]= *c++ ; Buf[2]= *c++ ; Buf[3]= *c;
#endif
/*  DEBUG(("Buf = %x%x%x%x", Buf[0],Buf[1],Buf[2],Buf[3])); */
  result = svcs_send(vcsfd, 1, &Buf[0]);
  }
  if (result < 0)
    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
/*   if (Count & 3)
    DEBUG(("Non word aligned Read from Link")); */
/*  DEBUG(("Sent nr.of.bytes to receive")); */
  result = svcs_recv(vcsfd, &RecvCount, Buffer); 
  if (result < 0)
    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
  if (!(((RecvCount << 2) == Count) || (Count < (RecvCount << 2))))
    ABORT(MISC_EXIT,(SE, "Unexpected RecvCount %d, Count %d", RecvCount, Count)); 
#endif
/*  DEBUG(("ReadLink request satisfied")); */
  return(Count);
}   




/*
 *   Write Link
 */
 
PUBLIC int WriteLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result; 
  INT32 SendCount;
  static char bufcpy[BOOT_BUFFER_LENGTH];
  BYTE *SendPtr = (BYTE *) Buffer;

#ifdef SVCS 
  
/*  if (Count & 3)
    DEBUG(( "Non word aligned Write to Link")); */
  if (Count & 3)
    SendCount = (Count >> 2) + 2;
  else 
    SendCount = (Count >> 2) + 1;
/*  DEBUG(("W Count= %d SendCount=%d", Count, SendCount)); 
  DEBUG(("Buffer[0..8] : %c%c%c%c%c%c%c%c",Buffer[0],Buffer[1],Buffer[2],
                         Buffer[3],Buffer[4],Buffer[5],Buffer[6],Buffer[7]));*/
  { register BYTE *c;
         c = (BYTE *)&Count;
#ifdef BIG_ENDIAN
         bufcpy[3]= *c++ ; bufcpy[2]= *c++ ; bufcpy[1]= *c++ ; bufcpy[0]= *c;
#else
         bufcpy[0]= *c++ ; bufcpy[1]= *c++ ; bufcpy[2]= *c++ ; bufcpy[3]= *c;
#endif
  memcpy(&bufcpy[4],Buffer,Count);
  }
/*  DEBUG(("bufcpy[0..11]: %02.x%02.x%02.x%02.x - %c%c%c%c%c%c%c%c",bufcpy[0],bufcpy[1],
          bufcpy[2],bufcpy[3],bufcpy[4],bufcpy[5],bufcpy[6],bufcpy[7],
          bufcpy[8],bufcpy[9],bufcpy[10],bufcpy[11])); */
  result = svcs_send(vcsfd, SendCount, bufcpy);
  if (result < 0)
    ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
#endif
  return(Count); 
}



PRIVATE int sbWrite(supfd, procno, fun, value)
int supfd;
int procno;
int fun;
int value;
{
    int rc, nw, rx;
    SuperMsg m;
    
    /*
     * Build up request message.
     */
    m.super_cmd = SUPER_SBWRITE;
    m.super_id = procno;
    m.super_p1 = fun;
    m.super_p2 = value;
    /*
     * Convert Host to Meiko (little-endian) bytesex.
     */
    svcs_htomlongs(&m, &m, SUPER_ENTRY);
    /*
     * Send it down the supervisor socket to the svcs daemon.
     */
    DEBUG(("Before pipesend to supfd %d", supfd));
    rc = svcs_pipesend(supfd, SUPER_ENTRY, &m);
    DEBUG(("After pipesend to supfd"));
    if (rc < 0) return(rc);
    /*
     * Receive reply.  I can't quite remember whether we ought to
     * do a select() before this, we do so just in case ...
     */
    rx = (1 << supfd);
    select(supfd+1, &rx, NULL, NULL, NULL);
    DEBUG(("After select to supfd"));
    rc = svcs_piperecv(supfd, &nw, &m);
    DEBUG(("After piperecv to supfd"));
    if (rc < 0) return(rc);
    svcs_mtohlongs(&m, &m, SUPER_ENTRY);
    /*
     * Check it succeeded (could have been bad processor no).
     */
    if (m.super_id < 0) return(-1);
    return(0);   
}

/*=======================================================================*/

PUBLIC int writeReset(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_RESET, val ? 0xff : 0);
}

 
/*=======================================================================*/

PUBLIC int writeAnalyse(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_ANALYSE, val ? 0xff : 0);
}

/*=======================================================================*/


/*
 *   Reset Link
 */
 
PUBLIC int ResetLink ( LinkId )
   LINK LinkId;
{
  if (writeReset(supfd, 1, 1)) return 0;
  if (writeReset(supfd, 1, 0)) return 0;
  else return 1;
}



/*
 *   Analyse Link
 */
 
PUBLIC int AnalyseLink ( LinkId )
   LINK LinkId;
{
  if (writeAnalyse(supfd, 1, 1)) return 0;
  if (writeAnalyse(supfd, 1, 0)) return 0;
  else return 1;
} 




/*
 *   Test Error
 */
 
PUBLIC int TestError ( LinkId )
   LINK LinkId;
{
  return(0);
}  




/*
 *   Test Read
 */
 
PUBLIC int TestRead ( LinkId )
   LINK LinkId;
{
  return(1);
}




/*
 *   Test Write
 */
 
PUBLIC int TestWrite ( LinkId )
   LINK LinkId;
{
  return(1);
}

/*
 *  Eof
 */
