
/*
 --   ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      svcslink.c
 --
 --      Link module for Meiko boards with /dev/mpn device driver
 --
 --      Copyright (c) H7 1990.
 --      All Rights Reserved.
 --
 --   ---------------------------------------------------------------------------
*/

#include <memory.h>
#include <stdio.h>
#include <fcntl.h>
#include "svcs.h"
#include "inmos.h"
#include "iserver.h"
#include "supercmd.h"

EXTERN LINK TheLink;
PUBLIC int vcsfd, supfd, CocoPops;
EXTERN BOOL VerboseSwitch;
EXTERN BOOL MKboot;
PUBLIC Resources theResources;
PUBLIC int vcsfd;
PUBLIC int supfd;
PUBLIC DomainSpec *theDomain;
PRIVATE BYTE *BufferBootFileName;

PRIVATE VOID SendSwitch();



/*
 *   Open Link
 */

PUBLIC LINK OpenLink ( Name )
   BYTE *Name;
{
  int result, cprfd;
  long FileSize;
  struct ProcessorSpec *p;

#ifdef SVCS
  theDomain = svcs_newspec();
  p= theDomain->d_spec;
  svcs_procspec(theDomain, T800, 1); /* processor which just forwards messages */
  p++;
  svcs_procspec(theDomain, T800, 8); /* processor doing all the work */
  p->p_memunits = 2 *MEGABYTES;
  p++;
  svcs_procspec(theDomain, T800, 4); /* processor for loading PROGRAMS on*/
  p->p_memunits = 1 *MEGABYTES;
  svcs_wirespec(theDomain, 0, 2, 1, 0);
  svcs_wirespec(theDomain, 1, 1, 2, 2); /* default OPS connection: -w 0.1.1.2 */
  if ((result = svcs_grab(theDomain, &vcsfd, &supfd, &theResources)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Grabbed %d processors", theResources.r_nprocs)) ;  
  DEBUG(("Processor 0 is %s", theResources.r_linkname));
  DEBUG(("Supv busid of worker is %d", theResources.r_proc[1].p_svbusid));
  if ( ( BufferBootFileName = (BYTE *)getenv("IBUFFER") ) == NULL )
    ABORT(MISC_EXIT,(SE, "No environment variable IBUFFER defined"));
  if ((cprfd = open(BufferBootFileName, O_RDONLY)) == -1)
    ABORT(MISC_EXIT,(SE, "Failed to open buffer boot file %s", BufferBootFileName));
  if ((result = svcs_bootcpr(vcsfd, cprfd)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Buffer process booted using %s", BufferBootFileName)) ;  
#endif

   return(1);
}




/*
 *   Close Link
 */

PUBLIC int CloseLink ( LinkId )
   LINK LinkId;
{
   return(1);
}




/*
 *   Read Link
 */
 
PUBLIC int ReadLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result;
  static int RecvCount;
  BYTE *RecvPtr = (BYTE *) Buffer;
  static BYTE RecvBuffer[TRANSACTION_BUFFER_SIZE];

  SendSwitch();
  DEBUG(("ReadLink requests %d bytes",Count)); 
  if (MKboot) {
#ifdef SVCS
	  { BYTE Buf[4]; register BYTE *c; 
	    RecvCount = -Count;
		 c = (BYTE *)&RecvCount;
#ifdef BIG_ENDIAN
		 Buf[3]= *c++ ; Buf[2]= *c++ ; Buf[1]= *c++ ; Buf[0]= *c;
#else
		 Buf[0]= *c++ ; Buf[1]= *c++ ; Buf[2]= *c++ ; Buf[3]= *c;
#endif
	/*  DEBUG(("Buf = %x%x%x%x", Buf[0],Buf[1],Buf[2],Buf[3])); */
	  result = svcs_send(vcsfd, 1, &Buf[0]);
	  }
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
	/*   if (Count & 3)
	    DEBUG(("Non word aligned Read from Link")); */
	/*  DEBUG(("Sent nr.of.bytes to receive")); */
	  result = svcs_recv(vcsfd, &RecvCount, Buffer); 
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
	  if (!(((RecvCount << 2) == Count) || (Count < (RecvCount << 2))))
	    ABORT(MISC_EXIT,(SE, "Unexpected RecvCount %d, Count %d", RecvCount, Count)); 
#endif
	/*  DEBUG(("ReadLink request satisfied")); */
	  return(Count);
  }
  else { /* now using mapped SP protocol for greater efficiency */
         /* SP protocol is INT16::[]BYTE */
     static int size_part=1; /* This is now a protocol with state, buffering 
				/* two writes as well as two reads */

     if (size_part) { /* meaning "are we waiting for the INT16 ? */
          int nr_bytes = 0, check_val;
          DEBUG(("Size part,before svcs_recv"));
	  result = svcs_recv(vcsfd, &RecvCount, RecvBuffer); /* do a complete protocol action : INT16::[]BYTE */
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
        
          DEBUG(("Size part,RdLnk,RecvCount = %d",RecvCount));
          DEBUG(("Size part,RdLnk,SIZE = %d %d",RecvBuffer[0],RecvBuffer[1]));
            /* extract the INT16 part */
	  { BYTE Buf[4]; register INT32 *c = (INT32 *) Buf; 

            
#ifdef BIG_ENDIAN
		 Buf[3] = RecvBuffer[0]; Buf[2] = RecvBuffer[1] ;
#else
		 Buf[0] = RecvBuffer[0]; Buf[1] =  RecvBuffer[1];
#endif
	  nr_bytes=*c; 
          nr_bytes=(256*RecvBuffer[1]) + RecvBuffer[0] + 2;
          DEBUG(("Size part,RdLnk,nr bytes = %d",nr_bytes));
          DEBUG(("Size part,RdLnk,RecvBuffer[2,3] = %d,%d",RecvBuffer[2],RecvBuffer[3]));
          }
          /* compare the actual nr of words received with the supposedly received words according to the nr of bytes received */
          check_val = (nr_bytes / 4);
          if (nr_bytes & 3) check_val +=1;

	  if (RecvCount != check_val)
	    ABORT(MISC_EXIT,(SE, "Unexpected RecvCount %d, check_val %d", RecvCount, check_val)); 
        /* copy Count bytes into Buffer */
        memcpy(Buffer, RecvBuffer, Count);
        /* now we store the nr.of bytes received */
          RecvCount = nr_bytes - Count;
          if (RecvCount != 0) {
              size_part = 0; /* now we are waiting for the request for the []BYTES*/
          }
	  return(Count);
     }
     else { /* we are now dealing with a request for the []BYTE */
          if (Count != RecvCount)
	    ABORT(MISC_EXIT,(SE, "Unexpected Count %d, RecvCount %d", Count, RecvCount)); 
          DEBUG(("Size slice,RdLnk,nr bytes = %d",RecvCount));
          /* copy buffer contents to Buffer */
          memcpy(Buffer, &RecvBuffer[8],Count);
          size_part = 1; /* now we are waiting for the size part again */
	  return(Count);
     }
  }
}   




/*
 *   Write Link
 */
 
PUBLIC int WriteLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result; 
  INT32 SendCount;
  static char bufcpy[BOOT_BUFFER_LENGTH];
  static BOOL wsize_part = 1;
  static INT32 WrCount;
  BYTE *SendPtr = (BYTE *) Buffer;

#ifdef SVCS 
  DEBUG(("Writelink entered"));
  SendSwitch();
  if (MKboot) {
	  
	/*  if (Count & 3)
	    DEBUG(( "Non word aligned Write to Link")); */
	  if (Count & 3)
	    SendCount = (Count >> 2) + 2;
	  else 
	    SendCount = (Count >> 2) + 1;
	/*  DEBUG(("W Count= %d SendCount=%d", Count, SendCount)); 
	  DEBUG(("Buffer[0..8] : %c%c%c%c%c%c%c%c",Buffer[0],Buffer[1],Buffer[2],
				 Buffer[3],Buffer[4],Buffer[5],Buffer[6],Buffer[7]));*/
	  { register BYTE *c;
		 c = (BYTE *)&Count;
#ifdef BIG_ENDIAN
		 bufcpy[3]= *c++ ; bufcpy[2]= *c++ ; bufcpy[1]= *c++ ; bufcpy[0]= *c;
#else
		 bufcpy[0]= *c++ ; bufcpy[1]= *c++ ; bufcpy[2]= *c++ ; bufcpy[3]= *c;
#endif
	  memcpy(&bufcpy[4],Buffer,Count);
	  }
	/*  DEBUG(("bufcpy[0..11]: %02.x%02.x%02.x%02.x - %c%c%c%c%c%c%c%c",bufcpy[0],bufcpy[1],
		  bufcpy[2],bufcpy[3],bufcpy[4],bufcpy[5],bufcpy[6],bufcpy[7],
		  bufcpy[8],bufcpy[9],bufcpy[10],bufcpy[11])); */
	  result = svcs_send(vcsfd, SendCount, bufcpy);
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
#endif
	  return(Count); 
  }
  else { /* We are now doing only INT16::[]BYTE protocol in two phases */
#ifdef SVCS 
      if (wsize_part) {
        /* save nr of bytes to write until a later time and return for now */
        WrCount = Buffer[0] + (Buffer[1] * 256) + 2;
        DEBUG(("Writelink Sizepart, WrCount = %d, Count = %d",WrCount, Count));
        if (WrCount = Count) { /* only one packet */
	  if (WrCount & 3)
	    SendCount = (WrCount >> 2) + 1;
	  else 
	    SendCount = (WrCount >> 2);
	  result = svcs_send(vcsfd, SendCount, Buffer);
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
        return (Count);
          
        }
        wsize_part = 0; /* We are now waiting for the []BYTE part */
        memcpy(bufcpy,Buffer,Count);
        return (Count);
      } 
      else {

        /* check if we are sending the amount of bytes specified in the 
           previous request */
        DEBUG(("Writelink Slicepart, Count = %d",Count));
        if (Count != (WrCount-2))
	    ABORT(MISC_EXIT,(SE, "Unexpected Count %d, WrCount %d", Count, WrCount)); 

	/*  if (Count & 3)
	    DEBUG(( "Non word aligned Write to Link")); */
	  if (WrCount & 3)
	    SendCount = (WrCount >> 2) + 1;
	  else 
	    SendCount = (WrCount >> 2);
	/*  DEBUG(("W Count= %d SendCount=%d", Count, SendCount)); 
	  DEBUG(("Buffer[0..8] : %c%c%c%c%c%c%c%c",Buffer[0],Buffer[1],Buffer[2],
				 Buffer[3],Buffer[4],Buffer[5],Buffer[6],Buffer[7]));*/
	  memcpy(&bufcpy[3],Buffer,Count);
	/*  DEBUG(("bufcpy[0..11]: %02.x%02.x%02.x%02.x - %c%c%c%c%c%c%c%c",bufcpy[0],bufcpy[1],
		  bufcpy[2],bufcpy[3],bufcpy[4],bufcpy[5],bufcpy[6],bufcpy[7],
		  bufcpy[8],bufcpy[9],bufcpy[10],bufcpy[11])); */
	  result = svcs_send(vcsfd, SendCount, bufcpy);
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
        return (Count);
    }
  }
#endif
}



PRIVATE VOID SendSwitch()
{
  static int PrevState=1;
  int result;

  if (PrevState != MKboot) { /* tell server to switch mode */
    
      int Count; 
      static INT32 Dummy[2] ={0,0};

      /* send packet with INT16 = 0 */
      DEBUG(("Before SendSwitch()"));
      result = svcs_send(vcsfd, 1, Dummy);
      if (result < 0)
        ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
      PrevState=MKboot;
      DEBUG(("After SendSwitch()"));
  }

}

PRIVATE int sbWrite(supfd, procno, fun, value)
int supfd;
int procno;
int fun;
int value;
{
    int rc, nw, rx;
    SuperMsg m;
    
    /*
     * Build up request message.
     */
    m.super_cmd = SUPER_SBWRITE;
    m.super_id = procno;
    m.super_p1 = fun;
    m.super_p2 = value;
    /*
     * Convert Host to Meiko (little-endian) bytesex.
     */
    svcs_htomlongs(&m, &m, SUPER_ENTRY);
    /*
     * Send it down the supervisor socket to the svcs daemon.
     */
    DEBUG(("Before pipesend to supfd %d", supfd));
    rc = svcs_pipesend(supfd, SUPER_ENTRY, &m);
    DEBUG(("After pipesend to supfd"));
    if (rc < 0) return(rc);
    /*
     * Receive reply.  I can't quite remember whether we ought to
     * do a select() before this, we do so just in case ...
     */
    rx = (1 << supfd);
    select(supfd+1, &rx, NULL, NULL, NULL);
    DEBUG(("After select to supfd"));
    rc = svcs_piperecv(supfd, &nw, &m);
    DEBUG(("After piperecv to supfd"));
    if (rc < 0) return(rc);
    svcs_mtohlongs(&m, &m, SUPER_ENTRY);
    /*
     * Check it succeeded (could have been bad processor no).
     */
    if (m.super_id < 0) return(-1);
    return(0);   
}

/*=======================================================================*/

PUBLIC int writeReset(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_RESET, val ? 0xff : 0);
}

 
/*=======================================================================*/

PUBLIC int writeAnalyse(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_ANALYSE, val ? 0xff : 0);
}

/*=======================================================================*/


/*
 *   Reset Link
 */
 
PUBLIC int ResetLink ( LinkId )
   LINK LinkId;
{
  if (writeReset(supfd, 1, 1)) return 0;
  if (writeReset(supfd, 1, 0)) return 0;
  else return 1;
}



/*
 *   Analyse Link
 */
 
PUBLIC int AnalyseLink ( LinkId )
   LINK LinkId;
{
  /* return(1); */
  if (writeAnalyse(supfd, 1, 1)) return 0;
  if (writeAnalyse(supfd, 1, 0)) return 0;
  else return 1;
} 




/*
 *   Test Error
 */
 
PUBLIC int TestError ( LinkId )
   LINK LinkId;
{
  return(0);
}  




/*
 *   Test Read
 */
 
PUBLIC int TestRead ( LinkId )
   LINK LinkId;
{
  return(1);
}




/*
 *   Test Write
 */
 
PUBLIC int TestWrite ( LinkId )
   LINK LinkId;
{
  return(1);
}

/*
 *  Eof
 */
