
/*
 --   ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      svcslink.c
 --
 --      Link module for Meiko boards with /dev/mpn device driver
 --
 --      Copyright (c) H7 1990.
 --      All Rights Reserved.
 --
 --   ---------------------------------------------------------------------------
*/

#include <memory.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <errno.h>
#include <svcs.h>
#include "inmos.h"
#include "iserver.h"
#include "supercmd.h"
#ifdef WIRE_PROC_SPEC
#include "types.h"
#include "claim.h"
#endif

EXTERN LINK TheLink;
EXTERN int CocoPops;
EXTERN BOOL VerboseSwitch;
PUBLIC int supfd;
EXTERN BOOL MKboot;
PUBLIC Resources theResources;
PUBLIC DomainSpec *theDomain;
#ifdef WIRE_PROC_SPEC
EXTERN PUBLIC List TheMatches;     /* The linked list of Matches          */
EXTERN PUBLIC List TheWires;       /* The linked list of wires            */
#endif
PRIVATE BYTE *BufferBootFileName;

PRIVATE VOID SendSwitch(/* LinkId */);



/*
 *   Open Link
 */

PUBLIC LINK OpenLink ( Name )
   BYTE *Name;
{
  int result, cprfd;
  long FileSize;
  struct ProcessorSpec *p;
  int vcsfd;

#ifdef SVCS
#ifdef WIRE_PROC_SPEC
  theDomain=claim(&TheMatches, &TheWires, TRUE, &vcsfd, &supfd);
  if(theDomain==NULL)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
#else  

  theDomain = svcs_newspec();
  p= theDomain->d_spec;
  svcs_procspec(theDomain, T800, 1); /* processor which just forwards messages */
  p++;
  svcs_procspec(theDomain, T800, 8); /* processor doing all the work */
  p->p_memunits = 2 *MEGABYTES;
  p++;
  svcs_procspec(theDomain, T800, 4); /* processor for loading PROGRAMS on*/
  p->p_memunits = 1 *MEGABYTES;
  svcs_wirespec(theDomain, 0, 2, 1, 0);
  svcs_wirespec(theDomain, 1, 1, 2, 2); /* default OPS connection: -w 0.1.1.2 */
  if ((result = svcs_grab(theDomain, &vcsfd, &supfd, &theResources)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Grabbed %d processors", theResources.r_nprocs)) ;  
  DEBUG(("Processor 0 is %s", theResources.r_linkname));
  DEBUG(("Supv busid of worker is %d", theResources.r_proc[1].p_svbusid));
#endif
  if ( ( BufferBootFileName = (BYTE *)getenv("IBUFFER") ) == NULL )
    ABORT(MISC_EXIT,(SE, "No environment variable IBUFFER defined"));
  if ((cprfd = open(BufferBootFileName, O_RDONLY)) == -1)
    ABORT(MISC_EXIT,(SE, "Failed to open buffer boot file %s", BufferBootFileName));
  if ((result = svcs_bootcpr(vcsfd, cprfd)) < 0)
    ABORT(MISC_EXIT,(SE,"%s",svcs_errmsg(svcs_errno)));
  DEBUG(( "Buffer process booted using %s", BufferBootFileName)) ;  
#endif

   return(vcsfd);
}




/*
 *   Close Link
 */

PUBLIC int CloseLink ( LinkId )
   LINK LinkId;
{
   close(LinkId);
   close(supfd);
   svcs_freespec(theDomain);
   return(1);
}




/*
 *   Read Link
 */
 
PUBLIC int ReadLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result;
  static int RecvCount;
  static BYTE RecvBuffer[TRANSACTION_BUFFER_SIZE];
  int hdr;
  struct iovec iov[3];

  SendSwitch(LinkId);
  DEBUG(("ReadLink requests %d bytes",Count)); 
  if (MKboot) {
          hdr = mtohINT32(-Count);
	  result = svcs_send(LinkId, 1, &hdr);
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
	/*  DEBUG(("Sent nr.of.bytes to receive")); */
	  result = svcs_recv(LinkId, &RecvCount, Buffer); 
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s\n", svcs_errmsg(svcs_errno)));
	  if (!(((RecvCount << 2) == Count) || (Count < (RecvCount << 2))))
	    ABORT(MISC_EXIT,(SE, "Unexpected RecvCount %d, Count %d", RecvCount, Count)); 
	/*  DEBUG(("ReadLink request satisfied")); */
	  return(Count);
  }
  else { /* now using mapped SP protocol for greater efficiency */
         /* SP protocol is INT16::[]BYTE */
     static int size_part=1; /* This is now a protocol with state, buffering 
				/* two writes as well as two reads */

     if (size_part) { /* meaning "are we waiting for the INT16 ?" */
          int nr_bytes = 0, check_val;
          DEBUG(("Size part,before svcs_recv"));
          /* split the received data up in a requested part and the rest */
          /* first the requested part */
          iov[0].iov_base = (caddr_t) &hdr ;
          iov[0].iov_len  = 4;
          iov[1].iov_base = (caddr_t) Buffer;
          iov[1].iov_len  = Count;
          /* then any remaining part of the SP transaction */
          iov[2].iov_base = (caddr_t) RecvBuffer;
          iov[2].iov_len  = TRANSACTION_BUFFER_SIZE;
          /* go and read them bytes ... */
          for (;;) {
              result = readv(LinkId, iov, 3);
              if ((result > 0) || (errno != EINTR)) break;
          }
          RecvCount = (result >= 4) ? mtohINT32(hdr) : -1;
          if (result != 4*(1+RecvCount)) return _svcserr(SVCS_ERECV);

          DEBUG(("Size part,RdLnk,RecvCount = %d",RecvCount));
          /* extract the INT16 part */

          nr_bytes=(256* (BYTE) Buffer[1]) + (BYTE) Buffer[0] + 2;
          DEBUG(("Size part,RdLnk,nr bytes = %d",nr_bytes));
          /* compare the actual nr of words received with the supposedly received words 
             according to the nr of bytes received */
          check_val = (nr_bytes >> 2);
          if (nr_bytes & 3) check_val +=1;

	  if (RecvCount != check_val)
	    ABORT(MISC_EXIT,(SE, "Unexpected RecvCount %d, check_val %d", RecvCount, check_val)); 
        /* now we store the nr.of bytes received */
          RecvCount = nr_bytes - Count;
          if (RecvCount != 0) {
              size_part = 0; /* now we are waiting for the request for the []BYTES*/
          }
	  return(Count);
     }
     else { /* we are now dealing with a request for the []BYTE */
          if (Count != RecvCount)
	    ABORT(MISC_EXIT,(SE, "Unexpected Count %d, RecvCount %d", Count, RecvCount)); 
          DEBUG(("Size slice,RdLnk,nr bytes = %d",RecvCount));
          /* copy buffer contents to Buffer */
          memcpy(Buffer, &RecvBuffer[0],Count);
          size_part = 1; /* now we are waiting for the size part again */
	  return(Count);
     }
  }
}   




/*
 *   Write Link
 */
 
PUBLIC int WriteLink ( LinkId, Buffer, Count, Timeout )
   LINK LinkId;
   char *Buffer;
   unsigned int Count;
   int Timeout;
{
  int result; 
  INT32 SendCount;
  static char bufcpy[BOOT_BUFFER_LENGTH];
  static BOOL wsize_part = 1;
  static unsigned INT32 WrCount;
  int hdr;
  struct iovec iov[3];

  DEBUG(("Writelink entered"));
  SendSwitch(LinkId);
  if (MKboot) {
	  
	  if (Count & 3)
	    SendCount = (Count >> 2) + 2;
	  else 
	    SendCount = (Count >> 2) + 1;
          /* specify total nr of words for transaction in transputer bytesex */
          hdr = mtohINT32(SendCount);
          /* specify total nr of bytes for transaction in transputer bytesex */
          WrCount = mtohINT32(Count);
          /* fill in control structure for writev() */
          iov[0].iov_base = (caddr_t) &hdr;
          iov[0].iov_len  = 4;
          iov[1].iov_base = (caddr_t) &WrCount;
          iov[1].iov_len  = 4;
          iov[2].iov_base = (caddr_t) Buffer;
          iov[2].iov_len  = Count;
          /* write to link until bytes are transferred or error */
          for (;;) {
              result = writev(LinkId, iov, 3);
              if ((result > 0) || (errno != EINTR)) break;
          }
          if (result != 8 + Count) return _svcserr(SVCS_ESEND);
	  return(Count); 
  }
  else { /* We are now doing only INT16::[]BYTE protocol in two phases */
#ifdef SVCS 
      if (wsize_part) {
        WrCount = (BYTE) Buffer[0] + ((BYTE) Buffer[1] * 256) + 2;
        /* save nr of bytes to write until a later time and return for now */
/*        DEBUG(("Writelink Sizepart, WrCount = %d, Count = %d",WrCount, Count)); */
        if (WrCount == Count) { /* only one packet */
	    if (WrCount & 3)
	        SendCount = (WrCount >> 2) + 1;
            else 
  	        SendCount = (WrCount >> 2);
        if (Count < 6) {
           int i;
           DEBUG(("Small write: "));
           for (i = 0;i < Count; i++) {
              DEBUG(("%x ", (BYTE) Buffer[i]));
           }
        }
	    result = svcs_send(LinkId, SendCount, Buffer);
	    if (result < 0)
	        ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
            return (Count);
          
        }
        INFO(("WriteLink: copying %d bytes to bufcpy",Count));
        wsize_part = 0; /* We are now waiting for the []BYTE part */
        memcpy(bufcpy,Buffer,Count);
        return (Count);
      } 
      else {

        /* check if we are sending the amount of bytes specified in the 
           previous request */
        DEBUG(("Writelink Slicepart, Count = %d",Count));
        if (Count != (WrCount-2))
	    ABORT(MISC_EXIT,(SE, "Unexpected Count %d, WrCount %d", Count, WrCount)); 


	  if (WrCount & 3)
	    SendCount = (WrCount >> 2) + 1;
	  else 
	    SendCount = (WrCount >> 2);
	/*  DEBUG(("W Count= %d SendCount=%d", Count, SendCount)); */
	  memcpy(&bufcpy[3],Buffer,Count);
	  result = svcs_send(LinkId, SendCount, bufcpy);
	  if (result < 0)
	    ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
        return (Count);
    }
  }
#endif
}



PRIVATE VOID SendSwitch(LinkId)
LINK LinkId;
{
  static int PrevState=1;
  int result;
/* #ifdef SENDSW */
  if (PrevState != MKboot) { /* tell server to switch mode */
    
      int Count; 
      static INT32 Dummy[2] ={0,0};

      /* send packet with INT16 = 0 */
      DEBUG(("Before SendSwitch()"));
      result = svcs_send(LinkId, 1, Dummy);
      if (result < 0)
        ABORT(MISC_EXIT,(SE, "%s", svcs_errmsg(svcs_errno)));
      PrevState=MKboot;
      DEBUG(("After SendSwitch()"));
  }
/* #endif */
}

PRIVATE int sbWrite(supfd, procno, fun, value)
int supfd;
int procno;
int fun;
int value;
{
    int rc, nw, rx;
    SuperMsg m;
    
    /*
     * Build up request message.
     */
    m.super_cmd = SUPER_SBWRITE;
    m.super_id = procno;
    m.super_p1 = fun;
    m.super_p2 = value;
    /*
     * Convert Host to Meiko (little-endian) bytesex.
     */
    svcs_htomlongs(&m, &m, SUPER_ENTRY);
    /*
     * Send it down the supervisor socket to the svcs daemon.
     */
    DEBUG(("Before pipesend to supfd %d", supfd));
    rc = svcs_pipesend(supfd, SUPER_ENTRY, &m);
    DEBUG(("After pipesend to supfd"));
    if (rc < 0) return(rc);
    /*
     * Receive reply.  I can't quite remember whether we ought to
     * do a select() before this, we do so just in case ...
     */
    rx = (1 << supfd);
    select(supfd+1, &rx, NULL, NULL, NULL);
    DEBUG(("After select to supfd"));
    rc = svcs_piperecv(supfd, &nw, &m);
    DEBUG(("After piperecv to supfd"));
    if (rc < 0) return(rc);
    svcs_mtohlongs(&m, &m, SUPER_ENTRY);
    /*
     * Check it succeeded (could have been bad processor no).
     */
    if (m.super_id < 0) return(-1);
    return(0);   
}

/*=======================================================================*/


PRIVATE int sbCheckError(supfd, procno)
int supfd;
int procno;
{
    int rc, nw, rx;
    SuperMsg m;
    
    /*
     * Build up request message.
     */
    m.super_cmd = SUPER_SBREAD;
    m.super_id = procno;
    m.super_p1 = FN_SOFTERROR;
    /*
     * Convert Host to Meiko (little-endian) bytesex.
     */
    svcs_htomlongs(&m, &m, SUPER_ENTRY);
    /*
     * Send it down the supervisor socket to the svcs daemon.
     */
    DEBUG(("Before pipesend to supfd %d", supfd));
    rc = svcs_pipesend(supfd, SUPER_ENTRY, &m);
    DEBUG(("After pipesend to supfd"));
    if (rc < 0) return(rc);
    /*
     * Receive reply.  I can't quite remember whether we ought to
     * do a select() before this, we do so just in case ...
     */
    rx = (1 << supfd);
    select(supfd+1, &rx, NULL, NULL, NULL);
    DEBUG(("After select to supfd"));
    rc = svcs_piperecv(supfd, &nw, &m);
    DEBUG(("After piperecv to supfd"));
    if (rc < 0) return(rc);
    svcs_mtohlongs(&m, &m, SUPER_ENTRY);
    /*
     * Check it succeeded (could have been bad processor no).
     */
    if (m.super_id < 0) return(-1);
    DEBUG(("Error = %d",m.super_id));
    return(m.super_id);   
}

/*=======================================================================*/

PUBLIC int writeReset(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_RESET, val ? 0xff : 0);
}

 
/*=======================================================================*/

PUBLIC int writeAnalyse(supfd, procno, val)
int supfd;
int procno;
int val;
{
    return sbWrite(supfd, procno, FN_ANALYSE, val ? 0xff : 0);
}

/*=======================================================================*/


/*
 *   Reset Link
 */
 
PUBLIC int ResetLink ( LinkId )
   LINK LinkId;
{
  if (writeReset(supfd, 1, 1)) return 0;
  if (writeReset(supfd, 1, 0)) return 0;
  else return 1;
}



/*
 *   Analyse Link
 */
 
PUBLIC int AnalyseLink ( LinkId )
   LINK LinkId;
{
  /* return(1); */
  if (writeAnalyse(supfd, 1, 1)) return 0;
  if (writeAnalyse(supfd, 1, 0)) return 0;
  else return 1;
} 




/*
 *   Test Error
 */
 
PUBLIC int TestError ( LinkId )
   LINK LinkId;
{
  /* return(!sbCheckError(supfd, 1)); */
  return(0);
}  




/*
 *   Test Read
 */
 
PUBLIC int TestRead ( LinkId )
   LINK LinkId;
{
  return(1);
}




/*
 *   Test Write
 */
 
PUBLIC int TestWrite ( LinkId )
   LINK LinkId;
{
  return(1);
}

/*
 *  Eof
 */
