/* **********************************************************************
 * types.h
 *
 * This file contains the constants, structures and typedefs used.
 *
 * (C) David Beckett 1991, University of Kent at Canterbury
 * **********************************************************************
 */

#ifndef TYPES_H

#define TYPES_H

#include "node.h"

/* **********************************************************************
 * Constants
 * **********************************************************************
 */

/* Arguments to this program */
#define TESTARGS "dhip:w:"

/* Argument part separator string */
#define ARGUMENT_SEPARATORS "/"

/* String length constants */
#define MAXMODULESPERMACHINE   5
#define MAXBOARDSPERMODULE   200
#define MAXPROCSPERBOARD      24 /* Be cautious */
#define MAXPROCSPERUSER      256 /* Be generous */

#define MAXBOARDTYPENUMBER    999   /* The largest MKxxx number allowed    */
#define MAXPROCTYPENUMBER     999   /* The largest Txxx number allowed     */
#define MAXPROCMEMSIZE      65536   /* The largest memory size allowed (K) */

/* Match defines - values that signify no matches*/
#define MATCH_CHAR_NONE '\0'
#define MATCH_INT_NONE  -1

/* Option string length constants */
#define MAXOPTIONSTRINGLEN 20
#define MAXOPTIONSHELPLEN  60

#define MAXUSERNAMELEN        10

#define WIRE_BUFFER_SIZE      50

/* **********************************************************************
 * Enumerated types
 * **********************************************************************
 */

/* Status return from main body parsing routines */
typedef enum
{
  STATUS_OK,
  STATUS_FAIL,
  STATUS_DONE
  } Status;

/* The choices (in the array of integers) as returned to user */

typedef enum
{
  CHOICE_HELP,
  CHOICE_DEBUG,
  CHOICE_MAJOR_MODE,
  CHOICE_MINOR_MODE,
  CHOICE_OUTPUT_FLAGS,
  CHOICE_INFO
  } Choice;

/* The actions that can be done for each option */

typedef enum
{
  OR_IT,
  SET_IT,
  TOGGLE_IT
  } Action;


/* Type used in argument parsing */
typedef enum
{
  S_BAD,
  S_FOUND,
  S_NOT_FOUND
} Search;

/* **********************************************************************
 * Stuctures and typedefs
 * **********************************************************************
 */

/* The Option structure, used to set the flags as needed, for the user */

typedef struct 
{
  char   string[MAXOPTIONSTRINGLEN]; /* Argument to recognise      */
  char   letter;                     /* Single letter version      */  
  Action action;                     /* What to do with it         */
  int    value;                      /* Value to OR/SET/...        */
  int    choice_index;               /* Choice flag to do this to  */
  char   help[MAXOPTIONSHELPLEN];    /* Help information for this  */
} Option;


/* Match structure for deciding what to print if arguments are present */

typedef struct
{
  Node node;
  int  board_type;              /* Board type, ie MKxxx number          */
  char proc_class;              /* Ie 'T' or ...? 'H'                   */
  int  proc_type;		/* Processor type, ie Txxx number       */
  int  proc_memory;		/* Memory of processor, in K            */
  int  module;                  /* Module number of this processor      */
  int  board;                   /* Board number of this processor       */
  int  processor;               /* Processor number of this processor   */
  int  process_id;              /* UNIX process running this            */
  char user_id[MAXUSERNAMELEN]; /* The user invocation that owns this   */  
  int  count;                   /* The number of times this match req'd */
} Match;

/* Wire structure for wiring domains */
typedef struct 
{
  Node node;
  int p0;
  int l0;
  int p1;
  int l1;
} Wire;

/* **********************************************************************
 * Data Structures of Constant structures
 * **********************************************************************
 */


#endif
